@@ -69,6 +69,22 @@ impl<T, U> IntoDuration<U> for T
 ///
 /// Types implementing `TimePoint` can have a `FloatDuration` computed between them
 /// via `float_duration_since` in either direction.
+///
+/// #Examples
+/// ```rust
+/// # fn perform_expensive_task() {}
+/// // Don't forget to use TimePoint.
+/// use float_duration::{TimePoint, FloatDuration};
+/// use std::time::Instant;
+///
+/// let start = Instant::now();
+/// perform_expensive_task();
+///
+/// // Computing a duration using `Instant` cannot fail, so it is safe to unwrap.
+/// let elapsed = Instant::now().float_duration_since(start).unwrap();
+///
+/// println!("Took {}.", elapsed);
+/// ```
 pub trait TimePoint<Rhs = Self> {
     /// The type returned if there is an error computing the duration.
     type Error;
@@ -206,6 +222,27 @@ impl FloatDuration {
         self.secs.is_sign_negative()
     }
 
+    /// Return a number that represents the sign of `self`.
+    ///
+    /// - 1.0 if the value is positive, `+0.0` or `INFINITY`
+    /// - -1.0 if the value is negative, `-0.0` or `NEG_INFINITY`
+    /// - `NAN` if the value is `NAN`
+    #[inline]
+    pub fn signum(&self) -> f64 {
+        self.secs.signum()
+    }
+
+    /// Return the maximum of two durations.
+    #[inline]
+    pub fn max(self, other: FloatDuration) -> FloatDuration {
+        FloatDuration { secs: self.secs.max(other.secs) }
+    }
+    /// Return the minimum of two durations.
+    #[inline]
+    pub fn min(self, other: FloatDuration) -> FloatDuration {
+        FloatDuration { secs: self.secs.min(other.secs) }
+    }
+
     /// Return a new `FloatDuration` with the minimum possible value.
     #[inline]
     pub fn min_value() -> FloatDuration {
@@ -240,6 +277,8 @@ impl FloatDuration {
     }
 
     /// Create a `FloatDuration` object from a `std::time::Duration`.
+    ///
+    /// Equivalent to using `FloatDuration::from(duration)`
     #[inline]
     pub fn from_std(duration: time::Duration) -> FloatDuration {
         FloatDuration::seconds((duration.as_secs() as f64) +
@@ -271,6 +310,8 @@ impl FloatDuration {
     /// precision if the duration is too large to be entirely represented as a single
     /// value. Thus, if the absolute value of the total number of nanoseconds is
     /// greater than `i64::MAX`, only millisecond precision will be captured.
+    ///
+    /// Equivalent to using `FloatDuration::from(duration)`
     #[inline]
     pub fn from_chrono(duration: chrono::Duration) -> FloatDuration {
         if let Some(nanos) = duration.num_nanoseconds() {
@@ -419,12 +460,14 @@ impl FromDuration<FloatDuration> for chrono::Duration {
 }
 
 impl From<time::Duration> for FloatDuration {
+    #[inline]
     fn from(from: time::Duration) -> FloatDuration {
         FloatDuration::from_std(from)
     }
 }
 #[cfg(feature = "chrono")]
 impl From<chrono::Duration> for FloatDuration {
+    #[inline]
     fn from(from: chrono::Duration) -> FloatDuration {
         FloatDuration::from_chrono(from)
     }
@@ -460,6 +503,7 @@ impl fmt::Display for FloatDuration {
 impl ops::Neg for FloatDuration {
     type Output = FloatDuration;
 
+    #[inline]
     fn neg(self) -> FloatDuration {
         FloatDuration { secs: -self.secs }
     }
@@ -468,6 +512,7 @@ impl ops::Neg for FloatDuration {
 impl ops::Add<FloatDuration> for FloatDuration {
     type Output = FloatDuration;
 
+    #[inline]
     fn add(self, rhs: FloatDuration) -> FloatDuration {
         FloatDuration { secs: self.secs + rhs.secs }
     }
@@ -475,6 +520,7 @@ impl ops::Add<FloatDuration> for FloatDuration {
 impl ops::Sub<FloatDuration> for FloatDuration {
     type Output = FloatDuration;
 
+    #[inline]
     fn sub(self, rhs: FloatDuration) -> FloatDuration {
         FloatDuration { secs: self.secs - rhs.secs }
     }
@@ -483,6 +529,7 @@ impl ops::Sub<FloatDuration> for FloatDuration {
 impl ops::Mul<f64> for FloatDuration {
     type Output = FloatDuration;
 
+    #[inline]
     fn mul(self, rhs: f64) -> FloatDuration {
         FloatDuration { secs: self.secs * rhs }
     }
@@ -490,6 +537,7 @@ impl ops::Mul<f64> for FloatDuration {
 impl ops::Mul<FloatDuration> for f64 {
     type Output = FloatDuration;
 
+    #[inline]
     fn mul(self, rhs: FloatDuration) -> FloatDuration {
         FloatDuration { secs: self * rhs.secs }
     }
@@ -497,6 +545,7 @@ impl ops::Mul<FloatDuration> for f64 {
 impl ops::Div<f64> for FloatDuration {
     type Output = FloatDuration;
 
+    #[inline]
     fn div(self, rhs: f64) -> FloatDuration {
         FloatDuration { secs: self.secs / rhs }
     }
@@ -504,33 +553,39 @@ impl ops::Div<f64> for FloatDuration {
 impl ops::Div<FloatDuration> for FloatDuration {
     type Output = f64;
 
+    #[inline]
     fn div(self, rhs: FloatDuration) -> f64 {
         self.secs / rhs.secs
     }
 }
 
 impl ops::AddAssign<FloatDuration> for FloatDuration {
+    #[inline]
     fn add_assign(&mut self, rhs: FloatDuration) {
         self.secs += rhs.secs;
     }
 }
 impl ops::SubAssign<FloatDuration> for FloatDuration {
+    #[inline]
     fn sub_assign(&mut self, rhs: FloatDuration) {
         self.secs -= rhs.secs;
     }
 }
 
 impl ops::MulAssign<f64> for FloatDuration {
+    #[inline]
     fn mul_assign(&mut self, rhs: f64) {
         self.secs *= rhs;
     }
 }
 impl ops::DivAssign<f64> for FloatDuration {
+    #[inline]
     fn div_assign(&mut self, rhs: f64) {
         self.secs /= rhs;
     }
 }
 impl Default for FloatDuration {
+    #[inline]
     fn default() -> FloatDuration {
         FloatDuration::zero()
     }
@@ -590,6 +645,7 @@ mod tests {
         assert_eq!(duration1.as_days(), 3.0 / 24.0);
         assert_eq!(duration1.as_milliseconds(), 180.0 * 60.0 * 1000.0);
         assert!(duration1.is_positive());
+        assert_eq!(duration1.signum(), 1.0);
 
         let duration2 = FloatDuration::milliseconds(55.0);
         assert_eq!(duration2.as_seconds(), 0.055);
@@ -617,6 +673,7 @@ mod tests {
         assert_eq!(duration4.as_minutes(), -3.0);
         assert_eq!(duration4.as_hours(), -0.05);
         assert!(duration4.is_negative());
+        assert_eq!(duration4.signum(), -1.0);
 
         assert_eq!(FloatDuration::years(2.0), FloatDuration::days(365.0 * 2.0));
     }
@@ -652,6 +709,29 @@ mod tests {
         assert!(inf.as_years().is_infinite());
         assert!(inf.as_microseconds().is_infinite());
         assert!(FloatDuration::hours(10.0) / FloatDuration::minutes(0.0) == f64::INFINITY);
+
+        let mut d1 = FloatDuration::seconds(5.0);
+        d1 += FloatDuration::minutes(10.0);
+        d1 *= 2.0;
+        assert_eq!(d1,
+                   FloatDuration::minutes(20.0) + FloatDuration::seconds(10.0));
+        d1 /= 2.0;
+        assert_eq!(d1,
+                   FloatDuration::minutes(10.0) + FloatDuration::seconds(5.0));
+        d1 -= d1;
+        assert_eq!(d1, FloatDuration::zero());
+    }
+
+    #[test]
+    fn test_min_max() {
+        assert_eq!(FloatDuration::minutes(5.0).max(FloatDuration::minutes(10.0)),
+                   FloatDuration::minutes(10.0));
+        assert_eq!(FloatDuration::minutes(5.0).min(FloatDuration::minutes(10.0)),
+                   FloatDuration::minutes(5.0));
+        assert_eq!(FloatDuration::zero().max(FloatDuration::seconds(1.0)),
+                   FloatDuration::seconds(1.0));
+        assert_eq!(FloatDuration::zero().min(FloatDuration::seconds(1.0)),
+                   FloatDuration::zero());
     }
 
     #[test]
@@ -659,27 +739,49 @@ mod tests {
         let duration1 = FloatDuration::minutes(5.0);
         let std_duration1 = duration1.to_std().unwrap();
         assert!(duration1.is_positive());
+        assert_eq!(duration1.signum(), 1.0);
         assert_eq!(std_duration1, time::Duration::new(300, 0));
         assert_eq!(FloatDuration::from_std(std_duration1), duration1);
 
         let duration2 = FloatDuration::hours(-2.0);
         assert!(duration2.is_negative());
+        assert_eq!(duration2.signum(), -1.0);
         assert!(!duration2.to_std().is_ok());
         let std_duration2 = (-duration2).to_std().unwrap();
         assert_eq!(std_duration2, time::Duration::new(3600 * 2, 0));
+        assert_eq!(FloatDuration::from(std_duration2), -duration2);
         assert_eq!(FloatDuration::from_std(std_duration2), -duration2);
 
         assert_eq!(FloatDuration::zero().to_std().unwrap(),
                    time::Duration::new(0, 0));
+        assert_eq!(FloatDuration::zero().signum(), 1.0);
+        assert!(FloatDuration::zero().is_positive());
         assert!(FloatDuration::nanoseconds(-1.0).to_std().is_err());
         assert!(FloatDuration::max_value().to_std().is_err());
 
         assert_eq!(FloatDuration::from_std(time::Duration::new(0, 1)),
                    FloatDuration::nanoseconds(1.0));
+        assert_eq!(FloatDuration::from(time::Duration::new(0, 1)),
+                   FloatDuration::nanoseconds(1.0));
         assert_eq!(FloatDuration::from_std(time::Duration::new(1, 1)),
                    FloatDuration::seconds(1.0) + FloatDuration::nanoseconds(1.0));
     }
 
+    #[test]
+    fn test_time_point_std() {
+        use std::time::{Instant, SystemTime, UNIX_EPOCH};
+
+        let i = Instant::now();
+        let s = SystemTime::now();
+        assert_eq!(i.float_duration_since(i), Ok(FloatDuration::zero()));
+        assert_eq!(s.float_duration_since(s).unwrap(), FloatDuration::zero());
+        assert!(Instant::now().float_duration_since(i).unwrap() > FloatDuration::zero());
+        assert!(SystemTime::now().float_duration_since(s).unwrap() > FloatDuration::zero());
+
+        assert!(SystemTime::now().float_duration_since(UNIX_EPOCH).unwrap() >
+                FloatDuration::years(30.0));
+    }
+
     #[test]
     fn test_display() {
         assert_eq!(format!("{}", FloatDuration::minutes(3.5)), "3.5 minutes");
@@ -726,7 +828,7 @@ mod tests {
     #[cfg(feature = "chrono")]
     #[test]
     fn test_chrono_conversion() {
-        assert_eq!(FloatDuration::from_chrono(chrono::Duration::minutes(10)),
+        assert_eq!(FloatDuration::from(chrono::Duration::minutes(10)),
                    FloatDuration::minutes(10.0));
         assert_eq!(FloatDuration::from_chrono(chrono::Duration::hours(72)),
                    FloatDuration::days(3.0));
@@ -736,7 +838,7 @@ mod tests {
                    FloatDuration::milliseconds(-20.0));
         assert_eq!(FloatDuration::from_chrono(chrono::Duration::zero()),
                    FloatDuration::zero());
-        assert_eq!(FloatDuration::from_chrono(chrono::Duration::hours(10000)),
+        assert_eq!(FloatDuration::from(chrono::Duration::hours(10000)),
                    FloatDuration::hours(10000.0));
         assert_eq!(FloatDuration::from_chrono(chrono::Duration::milliseconds(1i64 << 62)),
                    FloatDuration::milliseconds((1i64 << 62) as f64));
@@ -771,7 +873,7 @@ mod tests {
     #[cfg(feature = "serde")]
     #[test]
     fn test_serde() {
-        use serde_test::{Token, assert_tokens};
+        use serde_test::{Token, assert_tokens, assert_de_tokens};
 
         let duration = FloatDuration::seconds(1.5);
         assert_tokens(&duration, &[Token::F64(duration.as_seconds())]);
@@ -784,5 +886,17 @@ mod tests {
         let duration3 = FloatDuration::days(5.0) + FloatDuration::minutes(35.2) +
                         FloatDuration::milliseconds(100.0);
         assert_tokens(&duration3, &[Token::F64(duration3.as_seconds())]);
+
+        assert_tokens(&FloatDuration::seconds(-10.0), &[Token::F64(-10.0)]);
+
+        assert_de_tokens(&FloatDuration::seconds(30.0), &[Token::F32(30.0)]);
+    }
+
+    #[cfg(feature = "serde")]
+    #[should_panic]
+    #[test]
+    fn test_serde_integer_panic() {
+        use serde_test::{Token, assert_de_tokens};
+        assert_de_tokens(&FloatDuration::seconds(10.0), &[Token::I32(10)]);
     }
 }
